"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Actions = require("./actions");
var Debugger_1 = require("./actions/Debugger");
var interfaces_1 = require("./actions/interfaces");
var ActionFunction_1 = require("./functions/ActionFunction");
var minimize_xml_1 = require("./utils/minimize-xml");
var CCMLBuilder = (function () {
    function CCMLBuilder(json, _debugger) {
        this.json = json;
        this._debugger = _debugger;
        this._lastCallbackResponseName = "";
        this._actionConstructors = {
            CCML: this.buildCCML,
            COLLECT: this.buildCollect,
            CONFERENCE: this.buildConference,
            DETECT: this.buildDetect,
            DIAL: this.buildDial,
            FOREACH: this.buildForEach,
            FUNCTION: this.buildFunction,
            GOOGLEANALYTICS: this.buildGoogleAnalytics,
            GOTO: this.buildGoto,
            HANGUP: this.buildHangUp,
            IF: this.buildIf,
            LABEL: this.buildLabel,
            PAUSE: this.buildPause,
            PLAY: this.buildPlay,
            QUEUE: this.buildQueue,
            RECORD: this.buildRecord,
            RECORDMESSAGE: this.buildRecordMessage,
            REDIRECT: this.buildRedirect,
            REJECT: this.buildReject,
            SAY: this.buildSay,
            SESSIONDATA: this.buildSessionData,
            SMS: this.buildSMS,
            STOP: this.buildStop,
            TRANSFER: this.buildTransfer,
            VAR: this.buildVar,
            WEBHOOK: this.buildWebhook,
        };
        this._functions = [];
        this._actions = [];
        if (json.debugger === true && this._debugger === undefined) {
            this._debugger = new Debugger_1.Debugger();
        }
        this._lastCallbackResponseName = json.name || "";
        this.addActions.apply(this, json.actions);
        if (Array.isArray(json.functions)) {
            this.addFunctions.apply(this, json.functions);
        }
    }
    CCMLBuilder.prototype.getJSONActionsWithIDs = function () {
        return this.json;
    };
    CCMLBuilder.prototype.getActions = function () {
        var _this = this;
        if (this._debugger !== undefined) {
            var db_1 = this._debugger;
            return this._actions.reduce(function (p, c) { return p.concat({ type: interfaces_1.Type.Debugger, toCCML: db_1.toCCML.bind(_this._debugger, c) }, c); }, []);
        }
        else {
            return this._actions.slice(0);
        }
    };
    CCMLBuilder.prototype.addActions = function () {
        var _this = this;
        var data = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            data[_i] = arguments[_i];
        }
        data.forEach(function (d) {
            if (!d.type) {
                Object.keys(d)
                    .filter(function (v) { return v.trim()[0] === "$"; })
                    .forEach(function (v) { return _this._actions.push(new Actions.Var(v, d[v])); });
            }
            else {
                var action = _this.callActionConstructor(d);
                _this._actions.push(action);
                d.id = action.id;
            }
        });
        return this;
    };
    CCMLBuilder.prototype.getFunctions = function () {
        return this._functions.slice(0);
    };
    CCMLBuilder.prototype.addFunctions = function () {
        var _this = this;
        var funcs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            funcs[_i] = arguments[_i];
        }
        funcs.forEach(function (f) { return _this._functions.push(new ActionFunction_1.ActionFunction(f)); });
        return this;
    };
    CCMLBuilder.prototype.build = function (env, isModule) {
        var _this = this;
        if (env === void 0) { env = "dev"; }
        if (isModule === void 0) { isModule = false; }
        var ccml = [];
        if (!isModule) {
            ccml.push("<!DOCTYPE CCML SYSTEM \"http://" + env + ".shoutpoint.com/TelWeb/ccml.dtd\">");
            ccml.push("<CCML>");
            ccml.push("<SET varName=\"collect_keys\" value=\"\" export=\"true\"/>");
            ccml.push("<SET varName=\"recordings\" value=\"\" export=\"true\"/>");
        }
        if (this._debugger !== undefined) {
            this._debugger.jsonActions = this.getJSONActionsWithIDs();
            ccml.push(this._debugger.toCCML({ id: "", name: "", type: interfaces_1.Type.Debugger }));
            ccml.push("<SAY>Debugger On. Callback start.</SAY>");
        }
        ccml = ccml.concat(this._actions.map(function (a) {
            var type = Actions.Type[a.type].toUpperCase(), comments = JSON.stringify((a.comments || "").replace(/<[^>]+>/g, ""));
            if (_this._debugger !== undefined) {
                return _this._debugger.toCCML(a) + "\n<!-- " + type + " " + comments + " -->\n" + a.toCCML() + "\n";
            }
            return "\n<!-- " + type + " " + comments + " -->\n" + a.toCCML() + "\n";
        }));
        ccml = ccml.concat(this._functions.map(function (f) { return f.toCCML(); }));
        if (this._debugger !== undefined) {
            ccml.push("<SAY>Debugger. Callback end.</SAY>");
        }
        if (!isModule) {
            ccml.push("</CCML>");
        }
        return env !== "dev" ? minimize_xml_1.minimizeXML(ccml.join("")) : ccml.join("");
    };
    CCMLBuilder.prototype.eventCCMLBuilder = function (actions, name) {
        if (name === void 0) { name = "event"; }
        return new CCMLBuilder({ actions: actions, name: name }, this._debugger);
    };
    CCMLBuilder.prototype.callActionConstructor = function (d) {
        d.params = d.params || {};
        return this._actionConstructors[d.type.toUpperCase()].call(this, d);
    };
    CCMLBuilder.prototype.buildCCML = function (data) {
        var action = new Actions.CCML(data.params.ccml, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildCollect = function (data) {
        var action = new Actions.Collect(data.name);
        action.comments = data.comments || "";
        action.numDigits = data.params.num_digits || 1;
        action.finishOnKey = data.params.finish_on_key || "#";
        action.digitsOnly = !!data.params.digitsOnly;
        action.timeout = data.params.timeout || 5000;
        action.actions = this.eventCCMLBuilder(data.actions || [], action.name + "_event").getActions();
        return action;
    };
    CCMLBuilder.prototype.buildConference = function (data) {
        var _this = this;
        var action = new Actions.Conference(data.params.name, data.name);
        action.comments = data.comments || "";
        action.event = (function (evt) {
            switch (evt) {
                case "leave":
                    return Actions.ConferenceEvent.LEAVE;
                case "mute":
                    return Actions.ConferenceEvent.MUTE;
                case "unmute":
                    return Actions.ConferenceEvent.UNMUTE;
                case "hold":
                    return Actions.ConferenceEvent.HOLD;
                case "resume":
                    return Actions.ConferenceEvent.RESUME;
                default:
                    return Actions.ConferenceEvent.JOIN;
            }
        })(data.params.event || "join");
        action.autoHold = String(data.params.auto_hold).toLowerCase() === "true";
        action.joinMuted = String(data.params.join_muted).toLowerCase() === "true";
        action.muteOnKey = String(data.params.mute_on_key || "off").toLowerCase();
        action.unmuteOnKey = String(data.params.unmute_on_key || "off").toLowerCase();
        action.exitOnKey = String(data.params.exit_on_key || "off").toLowerCase();
        action.async = String(data.params.async).toLowerCase() === "true";
        if (typeof data.params.max_participants === "number") {
            action.maxParticipants = Math.max(1, data.params.max_participants);
        }
        action.announceOnJoin = data.params.announce_on_join === "beep" ? "beep" : "none";
        action.holdAudio = data.params.hold_audio || [];
        action.endOnExit = String(data.params.end_on_exit).toLowerCase() === "true";
        action.holdOnExit = String(data.params.hold_on_exit).toLowerCase() === "true";
        action.startOnJoin = String(data.params.start_on_join).toLowerCase() !== "false";
        action.holdOnJoin = String(data.params.hold_on_join).toLowerCase() === "true";
        if (data.events) {
            Object.keys(Actions.ConferenceOnEvent)
                .map(function (k) { return Actions.ConferenceOnEvent[k]; })
                .forEach(function (e) {
                action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions());
            });
        }
        return action;
    };
    CCMLBuilder.prototype.buildDetect = function (data) {
        var _this = this;
        var action = new Actions.Detect(data.name);
        action.comments = data.comments || "";
        action.triggerLiveInMS = data.params.trigger_live_in_ms || 3000;
        if (data.events) {
            Object.keys(Actions.DetectOnEvent)
                .map(function (k) { return Actions.DetectOnEvent[k]; })
                .forEach(function (e) { return action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions()); });
        }
        return action;
    };
    CCMLBuilder.prototype.buildDial = function (data) {
        var _this = this;
        var action = new Actions.Dial(data.params.no, data.name);
        action.comments = data.comments || "";
        action.callerIdNo = data.params.caller_id_no || "${apiNo}";
        action.detection = data.params.detection || "none";
        action.inboundAPI = data.params.inbound_api || "CallFlows";
        action.disableRedialBlock = data.params.disable_redial_block === true;
        action.maxCallTime = data.params.max_call_time || 86400;
        action.timeout = data.params.timeout || 50;
        action.overrideCallbackConfig = data.params.override_callback_config || "";
        if (data.session_data !== undefined) {
            action.sessionData = data.session_data;
        }
        if (data.events) {
            Object.keys(Actions.DialOnEvent)
                .map(function (k) { return Actions.DialOnEvent[k]; })
                .forEach(function (e) {
                action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions());
            });
        }
        return action;
    };
    CCMLBuilder.prototype.buildForEach = function (data) {
        var action = new Actions.ForEach(data.params.iterator, this.eventCCMLBuilder(data.then || [], data.name + "_if_event").getActions(), data.params.valueVar, data.params.indexVar, data.params.keyVar, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildFunction = function (data) {
        var action = new Actions.ActionFunction(data.name);
        action.comments = data.comments || "";
        action.args = data.params;
        return action;
    };
    CCMLBuilder.prototype.buildGoogleAnalytics = function (data) {
        var action = new Actions.GoogleAnalytics(data.callback_response, data.params, this._lastCallbackResponseName || data.name);
        action.comments = data.comments || "";
        if (data.event) {
            action.setGAEvent(data.event.action, data.event.label, data.event.category, data.event.value);
        }
        return action;
    };
    CCMLBuilder.prototype.buildGoto = function (data) {
        var action = new Actions.Goto(data.params.label, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildHangUp = function (data) {
        var action = new Actions.HangUp(data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildIf = function (data) {
        var action = new Actions.If(data.condition, this.eventCCMLBuilder(data.then || [], data.name + "_if_event").getActions(), data.name);
        action.comments = data.comments || "";
        if (Array.isArray(data.orElse)) {
            action.setEvent(Actions.IfEvent.OnElse, this.eventCCMLBuilder(data.orElse || [], action.name + "_else_event").getActions());
        }
        return action;
    };
    CCMLBuilder.prototype.buildLabel = function (data) {
        var action = new Actions.Label(data.name || "");
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildPause = function (data) {
        var action = new Actions.Pause(data.params.seconds, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildPlay = function (data) {
        var action = new Actions.Play(data.params.url || data.params.dtmf, data.name);
        action.comments = data.comments || "";
        if (!!data.params.url) {
            action.stopOnKey = String(data.params.stop_on_key).toLowerCase() === "true";
        }
        else {
            action.dtmfDelay = data.params.dtmf_delay || 250;
            action.dtmfDashDelay = data.params.dtmf_dash_delay || 250;
        }
        action.repeat = data.params.repeat || 1;
        action.parseVars = data.params.parse_vars !== false;
        return action;
    };
    CCMLBuilder.prototype.buildQueue = function (data) {
        var _this = this;
        var action = new Actions.Queue(data.params.name, data.name);
        action.comments = data.comments || "";
        action.event = (function (evt) {
            switch (evt) {
                case "poll":
                    return Actions.QueueEvent.POLL;
                case "poll_connect":
                    return Actions.QueueEvent.POLL_CONNECT;
                case "remove":
                    return Actions.QueueEvent.REMOVE;
                default:
                    return Actions.QueueEvent.OFFER;
            }
        })((data.params.event || "offer").toLowerCase());
        action.async = String(data.params.async).toLowerCase() === "true";
        action.maxCalls = typeof data.params.max_calls === "number" ? +data.params.max_calls : undefined;
        action.holdAudio = data.params.hold_audio !== undefined ? data.params.hold_audio : true;
        if (data.events) {
            Object.keys(Actions.QueueOnEvent)
                .map(function (k) { return Actions.QueueOnEvent[k]; })
                .forEach(function (e) {
                action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions());
            });
        }
        return action;
    };
    CCMLBuilder.prototype.buildRecord = function (data) {
        var action = new Actions.Record(data.params.action, data.params.group, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildRecordMessage = function (data) {
        var _this = this;
        var action = new Actions.RecordMessage(data.params.action, data.params.group, data.name);
        action.comments = data.comments || "";
        action.playBeep = data.params.play_beep !== false;
        action.finishOnKey = data.params.finish_on_key || "any";
        action.trimSilence = data.params.trimSilence !== false;
        action.timeout = data.params.timeout || 5;
        action.maxRecordingTime = data.params.max_recording_time || 30;
        if (data.events) {
            Object.keys(Actions.RecordMessageOnEvent)
                .map(function (k) { return Actions.RecordMessageOnEvent[k]; })
                .forEach(function (e) {
                action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions());
            });
        }
        return action;
    };
    CCMLBuilder.prototype.buildRedirect = function (data) {
        var action = new Actions.Redirect(data.params.url, data.name);
        action.comments = data.comments || "";
        action.method = data.params.method;
        action.contentType = data.params.content_type;
        action.headers = data.params.headers;
        return action;
    };
    CCMLBuilder.prototype.buildReject = function (data) {
        var action = new Actions.Reject(data.params && data.params.reason, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildSay = function (data) {
        var action = new Actions.Say(data.params.text, data.name);
        action.comments = data.comments || "";
        action.repeat = data.params.repeat || 1;
        action.stopOnKey = String(data.params.stop_on_key).toLowerCase() === "true";
        action.gender = data.params.gender || "default";
        action.language = data.params.language || "default";
        action.voice = data.params.voice || "default";
        action.parseVars = data.params.parse_vars !== false;
        return action;
    };
    CCMLBuilder.prototype.buildSessionData = function (data) {
        var action = new Actions.SessionData(data.session_data, data.name);
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildSMS = function (data) {
        var action = new Actions.SMS(data.params.message, data.name);
        action.no = data.params.no || "${callerNo}";
        action.callerIdNo = data.params.caller_id_no || "${apiNo}";
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildStop = function (data) {
        var action = new Actions.Stop(data.name);
        action.actions = (data.params.type || "AUDIO").toUpperCase();
        action.comments = data.comments || "";
        return action;
    };
    CCMLBuilder.prototype.buildTransfer = function (data) {
        var action = new Actions.Transfer(data.params.no, data.name);
        action.comments = data.comments || "";
        action.event = data.params.event || "start";
        action.callerIdNo = data.params.caller_id_no || "${callerNo}";
        action.async = String(data.params.async).toLowerCase() === "true";
        return action;
    };
    CCMLBuilder.prototype.buildVar = function (data) {
        var action = new Actions.Var(data.params.assignmentVar, data.params.expression, data.name);
        if (data.comments) {
            action.comments = data.comments;
        }
        return action;
    };
    CCMLBuilder.prototype.buildWebhook = function (data) {
        var _this = this;
        var action = new Actions.Webhook(data.params.event_name, data.name);
        action.comments = data.comments || "";
        if (data.params.url) {
            action.url = String(data.params.url);
        }
        if (data.params.content_type) {
            action.contentType = data.params.content_type;
        }
        if (data.params.headers) {
            action.headers = data.params.headers;
        }
        if (data.params.method) {
            action.method = data.params.method;
        }
        action.awaitResponse = !!data.params.await_response;
        action.webhookData = data.params.webhook_data || {};
        if (data.events) {
            Object.keys(Actions.WebhookOnEvent)
                .map(function (k) { return Actions.WebhookOnEvent[k]; })
                .forEach(function (e) {
                action.setEvent(e, _this.eventCCMLBuilder(data.events[e] || [], action.name + "_" + e + "_event").getActions());
            });
        }
        return action;
    };
    return CCMLBuilder;
}());
exports.CCMLBuilder = CCMLBuilder;
//# sourceMappingURL=CCMLBuilder.js.map